@@ -12,6 +12,7 @@
 */
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
 #include <errno.h>
 #include <syscall.h>
 #include <asm/unistd.h>
@@ -22,13 +23,17 @@
 static int def_NR_set_ipv4root = 227;
 #undef __NR_set_ipv4root
 
-static int __NR_set_ipv4root;
+static int __NR_set_ipv4root_rev0;
+static int __NR_set_ipv4root_rev1;
+static int rev_ipv4root=0;
 
-_syscall1(int, set_ipv4root, unsigned long, ip);
+_syscall1(int, set_ipv4root_rev0, unsigned long, ip);
+_syscall2(int, set_ipv4root_rev1, unsigned long, ip, unsigned long, bcast);
 
 static int def_NR_new_s_context = 226;
 #undef __NR_new_s_context
 static int __NR_new_s_context;
+static int rev_s_context=0;
 
 _syscall3(int, new_s_context, int, newctx, int, remove_cap, int, flags);
 
@@ -37,18 +42,27 @@ static void init()
 	static int is_init = 0;
 	if (!is_init){
 		FILE *fin = fopen ("/proc/self/status","r");
-		__NR_set_ipv4root = def_NR_set_ipv4root;
+		__NR_set_ipv4root_rev0 = def_NR_set_ipv4root;
+		__NR_set_ipv4root_rev1 = def_NR_set_ipv4root;
 		__NR_new_s_context = def_NR_new_s_context;
 		if (fin != NULL){
 			char line[100];
 			while (fgets(line,sizeof(line)-1,fin)!=NULL){
 				int num;
-				char title[100];
-				if (sscanf(line,"%s %d",title,&num)==2){
+				char title[100],rev[100];
+				rev[0] = '\0';
+				if (sscanf(line,"%s %d %s",title,&num,rev)>=2){
 					if (strcmp(title,"__NR_set_ipv4root:")==0){
-						__NR_set_ipv4root = num;
+						__NR_set_ipv4root_rev0 = num;
+						__NR_set_ipv4root_rev1 = num;
+						if (strncmp(rev,"rev",3)==0){
+							rev_ipv4root = atoi(rev+3);
+						}
 					}else if (strcmp(title,"__NR_new_s_context:")==0){
 						__NR_new_s_context = num;
+						if (strncmp(rev,"rev",3)==0){
+							rev_s_context = atoi(rev+3);
+						}
 					}
 				}
 			}
@@ -64,10 +78,14 @@ extern "C" int call_new_s_context(int newctx, int remove_cap, int flags)
 	return new_s_context(newctx,remove_cap,flags);
 }
 
-extern "C" int call_set_ipv4root (unsigned long ip)
+extern "C" int call_set_ipv4root (unsigned long ip, unsigned long bcast)
 {
 	init();
-	return set_ipv4root (ip);
+	if (rev_ipv4root == 0){
+		return set_ipv4root_rev0 (ip);
+	}else{
+		return set_ipv4root_rev1 (ip,bcast);
+	}
 }
 
 
